import json


class Point:
    def __init__(self, x: int = 0, y: int = 0):
        self.x = x
        self.y = y

    def __str__(self):
        return json.dumps({'x': self.x, 'y': self.y})

    def to_dict(self):
        return {'x': self.x, 'y': self.y}


class Color:
    def __init__(self, color: str):
        self.r = int(color[1:3], 16)
        self.g = int(color[3:5], 16)
        self.b = int(color[5:], 16)

    def to_rgb(self):
        return [self.r, self.g, self.b]


class Size:
    def __init__(self, width, height):
        self.width = width
        self.height = height

class Point:
    def __init__(self, x: int = 0, y: int = 0):
        self.x = x
        self.y = y

    def __str__(self):
        return json.dumps({'x': self.x, 'y': self.y})

    def to_dict(self):
        return {'x': self.x, 'y': self.y}


class OcrRes:
    def __init__(self, text: str = None, confidence: float = None, region_position: list = None):
        self.text = text
        self.confidence = float(confidence)
        self.region_position = region_position
        if self.region_position:
            self.center_x = region_position[2][0] - region_position[0][0]
            self.center_y = region_position[2][1] - region_position[0][1]


    def to_dict(self):
        return {'text': self.text, 'confidence': self.confidence, 'region_position': self.region_position}

    def __str__(self):
        return self.to_dict().__str__()




from typing import Any


class Intent:
    def __init__(self, action: str, uri: Any = None, context: Any = None,cls: Any = None):
        pass

    def __init__(self, packageContext: Any, cls: Any):
        pass

    @staticmethod
    def run(tag: str):
        """
        启动APP

        :param tag: 可以是包名,也可以是应用名称
        """
        pass

    @staticmethod
    def browser(tag: str):
        """

        """
        pass



import os.path
import subprocess
import zipfile

from PyQt5.QtCore import QThread, pyqtSignal
from airclick.windows.client import tools


class PkgThread(QThread):
    def __init__(self):
        super().__init__()
        self.package_path = None

    def set_package_path(self, package_path):
        self.package_path = package_path

    def run(self):
        req_txt_file = os.path.join(self.package_path, "requirements.txt")
        str_cmd = f"workon {tools.get_virtualenv_cureent()}\n pip freeze > {req_txt_file}\n"
        print(str_cmd)
        self.exec(str_cmd)

        try:
            zip_path = os.path.join(self.package_path, os.path.basename(self.package_path) + ".wac")

            with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
                for root, dirs, files in os.walk(self.package_path):
                    if ".idea" in root or '__pycache__' in root or '.git' in root:
                        print(root)
                        continue
                    for file in files:
                        file_path = os.path.join(root, file)

                        if os.path.splitext(file_path)[1] != ".wac":
                            zipf.write(file_path, os.path.relpath(file_path, self.package_path))

            os.startfile(self.package_path)
        except Exception as e:
            print(str(e))

    def exec(self, cmd: str):
        try:
            with subprocess.Popen("cmd", shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE) as proc:
                cmd = cmd.encode()
                proc.stdin.write(cmd)
                proc.stdin.flush()
                proc.stdin.close()

                infos = []
                errors = []
                for line in proc.stdout:
                    infos.append(line.strip())
                    # print(line.strip())
                    # self.log_info.emit(line.strip())

                for line in proc.stderr:
                    errors.append(line.strip())
                    # print(line.strip())
                    # self.log_error.emit(line.strip())

            return [proc.returncode, infos, errors]
        except Exception as e:
            print(str(e))


class PyEnvSet(QThread):
    set_statue = pyqtSignal(bool)

    def __init__(self):
        super().__init__()
        self.key = None
        self.value = None

    def setkv(self, key, value):
        self.key = key
        self.value = value

    def run(self):
        try:
            res = set_env(self.key, self.value)
            self.set_statue.emit(res)
        except Exception as e:
            print(str(e))


def set_env(key: str, value: str) -> bool:
    try:
        # Machine 是系统，User 是用户
        subprocess.run(['powershell',
                        f'[System.Environment]::SetEnvironmentVariable("{key}","{value}", [System.EnvironmentVariableTarget]::User)'])
        subprocess.run(['echo', f'%${key}%'], shell=True)
        os.environ[f'{key}'] = value
        return True
    except Exception as e:
        print(str(e))
        return False



from PIL.Image import Image

from airclick.windows.daos.screen import Point, Color


def find_colors(colors: str, sim: float, max_points: int, ore: int, img: Image, space: int) -> list:
    sim_space = 255 * (1 - sim)
    cs = colors.split('|')
    points = [None] * len(cs)
    for i in range(len(cs)):
        c = cs[i].split(',')
        if i == 0:
            points[i] = {'x': int(c[0]), 'y': int(c[1]), 'color': Color(c[2]).to_rgb()}
        else:
            points[i] = {'x': int(c[0]) - points[0]['x'], 'y': int(c[1]) - points[0]['y'],
                         'color': Color(c[2]).to_rgb()}

    return native_find_colors(sim_space, max_points, ore, img, points, space)


def compare_colors(colors: str, sim: float, img: Image):
    sim_space = 255 * (1 - sim)
    cs = colors.split('|')
    points = [None] * len(cs)
    for i in range(len(cs)):
        c = cs[i].split(',')
        points[i] = {'x': int(c[0]), 'y': int(c[1]), 'color': Color(c[2]).to_rgb()}

    for p in points:
        color = img.getpixel([p['x'], p['y']])
        if not compare_color(color, p['color'], sim_space):
            return False

    return True


def native_find_colors(sim: float, max_points: int, ore: int, img: Image, points: list, space: int) -> list:
    width = img.width
    height = img.height
    pixs = list(img.getdata())

    # print(len(pixs), width, height)

    fcs = []

    # print(ore)

    if ore == 1:
        for w in range(width):
            for h in range(height):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 2:
        for h in range(height):
            for w in range(width):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 3:
        for h in range(height):
            for w in reversed(range(width)):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 4:
        for w in reversed(range(width)):
            for h in range(height):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 5:
        for w in reversed(range(width)):
            for h in reversed(range(height)):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 6:
        for h in reversed(range(height)):
            for w in reversed(range(width)):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 7:
        for h in reversed(range(height)):
            for w in range(width):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    if ore == 8:
        for w in range(width):
            for h in reversed(range(height)):
                if check_color(width, height, pixs, w, h, points, sim):
                    fcs.append(Point(w, h))
                    if len(fcs) >= max_points != -1:
                        return fcs
                    space_image(pixs, space, width, height, w, h)

    return fcs


def space_image(pixs, space, width, height, w, h):
    x = w - space
    while x < w + space:
        x = x + 1
        if -1 < x < width:
            y = h - space
            while y < h + space:
                y = y + 1
                if x > -1 and y < height:
                    pixs[get_pix_forxy(width, x, y)] = [-1, -1, -1]


def check_color(width, height, pixs, x, y, points, sim):
    if compare_color(pixs[get_pix_forxy(width, x, y)], points[0]['color'], sim):

        i = 1
        while i < len(points):
            nx = points[i]['x'] + x
            ny = points[i]['y'] + y

            if nx < 0 or ny < 0 or nx >= width or ny >= height:
                return False

            if not (compare_color(pixs[get_pix_forxy(width, nx, ny)], points[i]['color'], sim)):
                return False

            i = i + 1

        return True

    return False


def get_pix_forxy(width, x, y):
    return width * y + x


def compare_color(c1, c2, sim):
    if c1[0] == -1:
        return False

    # print(c1,c2)

    if abs(c1[0] - c2[0]) <= sim and abs(c1[1] - c2[1]) <= sim and abs(c1[2] - c2[2]) <= sim:
        return True

    return False



from airclick.graphics import Point


def home():
    """
    Press the Home key.
    """
    pass


def back():
    """
    Press the Back key.
    """
    pass


def notifactions():
    """
    Open the notification shade.
    """
    pass


def lockscreen():
    """
    Turn the screen off and show the lock screen immediately.
    """
    pass


def screenshot():
    """
    Take a screenshot.
    """
    pass


def recents():
    """
    Open the recent apps view.
    """
    pass


class Catch:
    def __init__(self):
        pass

    def click(self) -> Point:
        """
        捕捉用户点击坐标

        返回结果:
        - Point: 点位对象，可通过 .x 和 .y 获取坐标
        """
        pass

    def msg(self, text: str) -> 'Catch':
        """
        设置捕获界面展示的信息

        参数:
        - text (str): 展示的文本信息

        返回结果:
        - Catch: Catch对象，用于链式调用其他方法
        """
        pass

    def shine(self, isShine: bool) -> 'Catch':
        """
        设置捕获界面是否闪屏

        参数:
        - isShine (bool): True表示闪屏，False表示不闪屏

        返回结果:
        - Catch: Catch对象，用于链式调用其他方法
        """
        pass





import json
import os.path
import re
import sys
from urllib.parse import unquote
import numpy
import pyautogui
import win32gui
from PIL import ImageGrab, Image

from airclick.windows.daos.screen import Size, Point, OcrRes
from airclick.windows.utils import screen_utils, airclickcv as aircv
import numpy as np
from airclick.windows.system import R
from airclick.windows.window import Window





def size(hwnd: int = None, rect=None) -> Image:
    temp_img = None
    if hwnd != 0 and hwnd is not None:
        psize = pyautogui.size()
        return Size(psize.width, psize.height)
    else:
        win = Window(hwnd)
        return Size(win.width, win.height)


def capture(hwnd: int = None, rect=None) -> Image:
    temp_img = None
    if hwnd != 0 and hwnd is not None:
        temp_rect = win32gui.GetWindowRect(hwnd)
        temp_img = ImageGrab.grab(temp_rect)
    else:
        temp_img = ImageGrab.grab()

    if rect:
        temp_img = temp_img.crop(rect)
    return temp_img


def get_color(x, y):
    img = capture()
    return img.getpixel((x, y))


def find_colors(colors: str = None, rect=[], sim: float = 0.9, ore: int = 2, space: int = 5, file: str = None,res_num:int=1):
    if res_num>1:
        return FindColors(colors, rect, sim, ore, space, file).find()
    else:
        return FindColors(colors, rect, sim, ore, space, file).find_all(res_num)


def compare_colors( colors: str = None, sim: float = 0.9, img_file: str = None):
        return CompareColors(colors,sim,img_file)

def cor(rect=[], pattern: str = None, confidence: float = 0.1, img_file: str = None,res_num:int =0):
    if res_num ==1:
        return Ocr(rect,pattern=pattern,confidence= confidence,img_path= img_file).find()
    else:
        return Ocr(rect, pattern=pattern, confidence=confidence, img_path=img_file).find_all(res_num)

def find_images(img_part: str = None, rect=None, source_file: str = None, confidence: float = 0.1,res_num:int=1):
    findimg = FindImages(img_part= img_part,source_rect=rect,source_file=source_file,confidence=confidence)
    if res_num==1:
        return findimg.find()
    else:
        return findimg.find_all()

def find_images_template(img_part: str = None, source_rect=None, source_file: str = None, confidence: float = 0.1,res_num:int=1, threshold=0.5, rgb=False, bgremove=False):
    findimg = FindImages(img_part= img_part,source_rect=source_rect,source_file=source_file,confidence=confidence)
    if res_num==1:
        return findimg.find_template(threshold= threshold,bgremove=bgremove,rgb=rgb)
    else:
        return findimg.find_all_template(maxcnt=res_num)

def find_images_sift(img_part: str = None, rect=None, source_file: str = None, confidence: float = 0.1,res_num:int=1, min_match_count: int = 4):
    findimg = FindImages(img_part= img_part,source_rect=rect,source_file=source_file,confidence=confidence)
    if res_num==1:
        return findimg.find_sift(min_match_count= min_match_count)
    else:
        return findimg.find_all_sift(min_match_count= min_match_count)

class FindColors:
    def __init__(self, colors: str = None, rect=[], sim: float = 0.9, ore: int = 2, space: int = 5, file: str = None,
                 dict_p: dict = None):
        self.rect = rect
        self.colors = colors
        self.sim = sim
        self.ore = ore
        self.space = space
        self.res_num = 1
        self.file = file

        if dict_p:
            if 'colors' in dict_p:
                self.colors = dict_p['colors']

            if 'rect' in dict_p:
                self.rect = dict_p['rect']

            if 'sim' in dict_p:
                self.sim = float(dict_p['sim'])

            if 'ore' in dict_p:
                self.ore = int(dict_p['ore'])

            if 'space' in dict_p:
                self.space = int(dict_p['space'])

            if 'file' in dict_p:
                self.file = unquote(dict_p['file'])

            if 'res_num' in dict_p:
                self.res_num = int(dict_p['res_num'])

    def find(self):
        self.res_num = 1
        res = self.exec_find()
        if res and len(res) > 0:
            return res[0]
        return None

    def find_all(self, res_num=sys.maxsize):
        self.res_num = res_num
        res = self.exec_find()
        if res or len(res) > 0:
            return res
        return None

    def exec_find(self):
        # 获取截图
        img = None
        if self.file:
            img = Image.open(self.file)
        else:
            img = ImageGrab.grab()

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        # 回填 坐标
        ps = screen_utils.find_colors(self.colors, self.sim, self.res_num, self.ore, img, self.space)
        if ps and self.rect and len(self.rect) == 4:
            for i in range(len(ps)):
                ps[i] = Point(ps[i].x + self.rect[0], ps[i].y + self.rect[1])

        return ps


class CompareColors:
    def __init__(self, colors: str = None, sim: float = 0.9, img_file: str = None, dict_p: dict = None):
        self.colors = colors
        self.sim = sim
        self.img_file = img_file
        if dict_p:
            if 'colors' in dict_p:
                self.colors = dict_p['colors']

            if 'sim' in dict_p:
                self.sim = float(dict_p['sim'])

            if 'path' in dict_p:
                self.img_file = unquote(dict_p['path'])

    def compare(self):
        if self.img_file:
            img = Image.open(self.img_file)
        else:
            img = capture()

        return screen_utils.compare_colors(self.colors, self.sim, img)


class Ocr:
    def __init__(self, rect=[], pattern: str = None, confidence: float = 0.1, img_path: str = None,
                 dict_p: dict = None):

        self.rect = rect
        self.pattern = pattern
        self.confidence = confidence
        self.img_path = img_path
        # self.ocr = PaddleOCR(ocr_version='PP-OCRv3')

        if dict_p:
            if 'rect' in dict_p:
                self.rect = dict_p['rect']

            if 'pattern' in dict_p:
                self.pattern = dict_p['pattern']

            if 'img_path' in dict_p:
                self.img_path = unquote(dict_p['img_path'])

            if 'confidence' in dict_p:
                self.sim = float(dict_p['confidence'])

            if 'findtype' in dict_p:
                if dict_p['findtype'] == 'find':
                    self.res_num = 1
                else:
                    self.res_num = 0

        # from easyocr import Reader
        #
        # self.ocr = Reader(['en', 'ch_sim'])

        from paddleocr import PaddleOCR
        self.ocr = PaddleOCR()

    def find(self) -> OcrRes:
        self.res_num = 1
        res = self.exec_find_paddle()
        if res:
            return res[0]
        return None

    def find_all(self, res_num: int = 0) -> [OcrRes]:
        self.res_num = res_num
        res = self.exec_find_paddle()
        return res

    def exec_find(self):
        if self.img_path:
            img = Image.open(self.img_path)
        else:
            img = capture(rect=self.rect)

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        # res_source = self.ocr.readtext(np.array(img))

        res_source = self.ocr.ocr(np.array(img))

        print(res_source)

        res = []
        if res_source:
            for r in res_source:
                if self.pattern:
                    pattern = re.compile(self.pattern)
                if r[2] >= self.confidence and (self.pattern is None or pattern.match(r[1])):

                    offsetx = 0
                    offsety = 0
                    if self.rect:
                        offsetx = self.rect[0]
                        offsety = self.rect[1]

                    region_position = r[0]
                    region_position = [[int(region_position[0][0]) + offsetx, int(region_position[0][1]) + offsety],
                                       [int(region_position[1][0]) + offsetx, int(region_position[1][1]) + offsety],
                                       [int(region_position[2][0]) + offsetx, int(region_position[2][1]) + offsety],
                                       [int(region_position[3][0]) + offsetx, int(region_position[3][1]) + offsety]]

                    if self.res_num == 0:
                        res.append(OcrRes(text=r[1], confidence=r[2], region_position=region_position))
                    elif len(res) < self.res_num:
                        res.append(OcrRes(text=r[1], confidence=r[2], region_position=region_position))
                    else:
                        return res

        return res

    def exec_find_paddle(self):
        if self.img_path:
            img = Image.open(self.img_path)
        else:
            img = capture(rect=self.rect)

        if self.rect and len(self.rect) == 4:
            img = img.crop(self.rect)

        res_source = self.ocr.ocr(np.array(img))
        res = []
        if res_source and res_source[0]:
            for r in res_source[0]:

                if self.pattern:
                    pattern = re.compile(self.pattern)
                    # [ [[238.0, 12.0], [330.0, 12.0], [330.0, 40.0], [238.0, 40.0]], ('版本控制', 0.9998359680175781)]

                if r[1][1] >= self.confidence and (self.pattern is None or pattern.match(r[1][0])):

                    offsetx = 0
                    offsety = 0
                    if self.rect:
                        offsetx = int(self.rect[0])
                        offsety = int(self.rect[1])

                    region_position = r[0]
                    region_position = [[int(region_position[0][0]) + offsetx, int(region_position[0][1]) + offsety],
                                       [int(region_position[1][0]) + offsetx, int(region_position[1][1]) + offsety],
                                       [int(region_position[2][0]) + offsetx, int(region_position[2][1]) + offsety],
                                       [int(region_position[3][0]) + offsetx, int(region_position[3][1]) + offsety]]

                    if self.res_num == 0:
                        res.append(OcrRes(text=r[1][0], confidence=r[1][1], region_position=region_position))
                    elif len(res) < self.res_num:
                        res.append(OcrRes(text=r[1][0], confidence=r[1][1], region_position=region_position))
                    else:
                        return res

        return res

    def to_dict(self):
        pass


class FindImages:

    def __init__(self, img_part: str = None, source_rect=None, source_file: str = None, confidence: float = 0.1,
                 dict_p: dict = None):

        self.img_part = img_part
        self.source_rect = source_rect
        self.source_file = source_file
        self.confidence = confidence

        if dict_p:
            if 'image' in dict_p:
                self.img_part = dict_p['image']

            if 'screen' in dict_p:
                self.source_file = unquote(dict_p['screen'])

            if 'sim' in dict_p:
                self.confidence = float(dict_p['sim'])
        else:
            if not os.path.exists(self.img_part):
                self.img_part = R().file("res/img/" + img_part)
                if not os.path.exists(self.img_part):
                    raise ValueError('图片路径不存在:' + self.img_part)

        self.source_img = numpy.array(self.__get_source_img())
        self.search_img = numpy.array(Image.open(self.img_part))

        # self.source_img = cv2.imread("D:\\work\\test\\ac_client\\assets\\tools\\capture\\1703126140.png")
        # self.search_img = cv2.imread("D:\\work\\test\\res\\img\\c.png")

    def find(self, various_size=False):

        res = self.find_template()
        if res:
            return res

        res = self.find_sift()

        return res

    def find_all(self, maxcnt: int = 0, various_size=False):
        res = self.find_all_template()
        if res and len(res) > 0:
            return res

        res = self.find_all_sift()
        return res

    def find_sift(self, min_match_count: int = 4):
        res = self.find_all_sift(maxcnt=1)
        if not res:
            return None
        return res[0]

    def find_all_sift(self, min_match_count: int = 4, maxcnt=0):
        res = aircv.find_all_sift(self.source_img, self.search_img, min_match_count, maxcnt)

        f_res = []
        for r in res:
            if r['confidence'] >= self.confidence:
                f_res.append(r)

        return f_res

    # def find_all_sift_gpt(self, min_match_count: int = 4, maxcnt=0):
    #     return aircv.find_all_sift_gpt(self.source_img, self.search_img, min_match_count, maxcnt)

    def find_template(self, threshold=0.5, rgb=False, bgremove=False):
        f_res = self.find_all_template(maxcnt=1, threshold=threshold, rgb=rgb, bgremove=bgremove)
        if f_res and len(f_res) > 0:
            return f_res[0]
        return None

    def find_all_template(self, maxcnt=0, threshold=0.5, rgb=False, bgremove=False):

        res = aircv.find_all_template(self.source_img, self.search_img, threshold, maxcnt, rgb, bgremove)

        f_res = []
        for r in res:
            if r['confidence'] >= self.confidence:
                f_res.append(r)

        return f_res

    def __get_source_img(self):
        temp_img = None
        if self.source_file:
            temp_img = Image.open(self.source_file)
        else:
            temp_img = capture()

        if self.source_rect and self.source_img.size == 4:
            temp_img = temp_img.resize(self.source_rect)

        return temp_img



import pyautogui
import pyperclip

LEFT = "left"
MIDDLE = "middle"
RIGHT = "right"
PRIMARY = "primary"
SECONDARY = "secondary"


def click(x=None, y=None, clicks=1, interval=0.0, button=PRIMARY, duration=0.02, tween=pyautogui.linear,
          logScreenshot=None, _pause=True):
    pyautogui.click(x, y, clicks, interval, button, duration, tween, logScreenshot, _pause)


def scroll(lines: int, x: int = None, y: int = None, h: bool = False, v: bool = True):
    if x and y:
        pyautogui.moveTo(x, y)

    if h:
        pyautogui.hscroll(lines, x, y)
    elif v:
        pyautogui.vscroll(lines, x, y)
    else:
        pyautogui.scroll(lines, x, y)


def drag(start_x: int, start_y: int, to_x: int, to_y: int, duration: float = 0.5):
    pyautogui.moveTo(start_x, start_y)
    pyautogui.dragTo(to_x, to_y, duration)


def mouse_down(x: int, y: int, button: str = PRIMARY, duration: float = 0.5, tween=pyautogui.linear):
    pyautogui.mouseDown(x, y, button, duration, tween)


def mouse_move(x: int, y: int, duration: float = 0.5, tween=pyautogui.linear):
    pyautogui.moveTo(x, y, duration, tween)


def mouse_up(x: int, y: int, button: str = PRIMARY, duration: float = 0.5, tween=pyautogui.linear):
    pyautogui.mouseUp(x, y, button, duration, tween)


def input(msg: str, interval=0.0):
    if any(0x7F < ord(c) < 0x10000 for c in msg):
        # 包含中文
        pyperclip.copy(msg)
        pyautogui.hotkey('ctrl', 'v')
    else:
        pyautogui.typewrite(msg, interval)


def key(keys, presses=1, interval=0.0):
    pyautogui.press(keys, presses, interval)


def key_hot(*args, **kwargs):
    pyautogui.hotkey(args, kwargs)


def key_down(key: str):
    pyautogui.keyDown(key)


def key_up(key:str):
    pyautogui.keyUp(key)



import ctypes
import json
import os.path
import signal
import subprocess
import threading
import pyautogui
import win32api
import win32con
import win32gui
from PyQt5.QtCore import Qt, QThread, pyqtSignal, QTimer, QObject, QEvent
from PyQt5.QtGui import QCursor, QPixmap, QIcon, QEnterEvent, QKeySequence
from PyQt5.QtWidgets import QApplication, QFrame, QFileDialog, QDialog, QHBoxLayout, QLabel, QCommandLinkButton, \
    QSpacerItem

from PyQt5.uic import loadUiType, loadUi
import sys

from airclick.windows import window
from airclick.windows.client import tools
import configparser
from airclick.windows.client.tools import EnvThread
from airclick.windows.client.worker import PkgThread, PyEnvSet

# Load the UI file
current_dir = os.path.dirname(os.path.realpath(sys.argv[0]))
# current_dir = os.path.dirname(__file__)
ui_bar_path = os.path.join(current_dir, "assets\pyui/bar.ui")
ui_step_path = "assets/pyui/step.ui"
ui_workspace_path = "assets/pyui/workspace.ui"
ui_pycharm_path = "assets/pyui/pycharminstall.ui"
ui_pyinstaller_path = os.path.join(current_dir, "assets/pyui/pyinstall.ui")
ui_home_path = "assets/pyui/home.ui"
ui_log_path = "assets/pyui/log.ui"
ui_app_item_path = "assets/pyui/item_app.ui"
ui_config_path = os.path.join(current_dir, "assets/pyui/home.ui")
ui_hwnd_path = os.path.join(current_dir, "assets/pyui/hwnd.ui")

py_388_path = "assets/data/python-3.8.8.exe"

py_work_config = "assets/tools/config.init"

py_apps_quick_start = ui_pyinstaller_path = os.path.join(current_dir, "assets/tools/apps")


# ui_img_bar = [os.path.join(current_dir,"assets/img/ico_run.png"),os.path.join(current_dir,"assets/img/ico_stop.png")]


class AirIcon(QIcon):
    def __init__(self, name: str):
        path = os.path.join(current_dir, "assets/img", name)
        super().__init__(path)


class BarWindow(QFrame):
    _instance = None
    log_window = None

    @classmethod
    def get_instance(cls, *args, **kwargs):
        return cls._instance

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)

        return cls._instance

    def __init__(self):
        super().__init__()
        self.is_dragging = False
        loadUi(ui_bar_path, self)
        try:
            # self.home_window = HomeWindow()
            self.config_window = None
            self.hwnd_window = HwndWindow()
        except Exception as e:
            print(str(e))
        self.setWindowFlag(Qt.WindowStaysOnTopHint)  # 悬浮在最顶部
        self.setWindowFlag(Qt.FramelessWindowHint)  # 隐藏标题栏
        self.setAttribute(Qt.WA_TranslucentBackground)  # 背景透明
        self.capture.clicked.connect(self.btn_capture)
        self.tools.clicked.connect(tools.tools_open)
        self.config.clicked.connect(self.btn_config)
        # self.logs.clicked.connect(self.btn_logs)
        # self.app.clicked.connect(self.btn_local_apps)
        self.btn_min.clicked.connect(self.btn_min_click)
        self.btn_close.clicked.connect(self.btn_close_click)
        self.hwnd.clicked.connect(self.btn_hwnd_click)

        self.ico.mousePressEvent = self.ico_down_event
        self.ico.mouseMoveEvent = self.ico_move_event
        self.ico.mouseReleaseEvent = self.ico_up_event
        self.menulabels = []
        self.menubuttons = [self.capture, self.tools, self.config, self.btn_min, self.btn_close,
                            self.labele_quick, self.hwnd]
        self.move_thread = None

        # self.run.clicked.connect()

        self.capture_context_num = 0

        self.auto_close_menu_time = 5
        self.start_auto_move_side_logic()

        # self.run_satue = [AirIcon("ico_run.png"), AirIcon("ico_stop.png")]

        w, h = pyautogui.size()
        self.move(w - 160, h / 2 - 170)

        self.enterEvent(None)

        # 快捷按键 与hwnd

        quick_keys = [
            {'tip': "CTRL+F1", 'keys': [win32con.VK_CONTROL, win32con.VK_F1], 'btn': 'capture',
             'fun': self.btn_capture},
            {'tip': "CTRL+F2", 'keys': [win32con.VK_CONTROL, win32con.VK_F2], 'btn': 'tools',
             'fun': tools.tools_open},
            {'tip': "CTRL+F3", 'keys': [win32con.VK_CONTROL, win32con.VK_F3], 'btn': 'hwnd',
             'fun': self.btn_hwnd_click},
        ]
        self.quick_keys = quick_keys

        self.qkey_timer = QTimer(self)
        self.qkey_timer.timeout.connect(self.on_qkey_all)
        self.qkey_timer.start(100)

        labele_quick = self.labele_quick

        def menu_enterEvent(self, event):
            btn_name = self.objectName()
            for key in quick_keys:
                if key['btn'] == btn_name:
                    labele_quick.setText(key['tip'])

        def menu_leaveEvent(self, event):
            labele_quick.setText("")

        for menu_btn in self.menubuttons:
            menu_btn.enterEvent = menu_enterEvent.__get__(menu_btn, QCommandLinkButton)
            menu_btn.leaveEvent = menu_leaveEvent.__get__(menu_btn, QCommandLinkButton)

    def btn_enterEvent(self, event=None):
        print(self.objectName())

    def on_qkey_all(self):
        for key_dao in self.quick_keys:
            true_key = True
            for key in key_dao['keys']:
                if not win32api.GetAsyncKeyState(key):
                    true_key = False
                    break
            if true_key:
                key_dao['fun']()

        if self.hwnd_window.isVisible():
            active_win = window.find()
            print(active_win)
            self.hwnd_window.change_hwnd(active_win)

        # if win32api.GetAsyncKeyState(win32con.VK_CONTROL) and win32api.GetAsyncKeyState(win32con.VK_F1):
        #     # CTRL+F1 ,截图
        #     self.btn_capture()
        #
        # if win32api.GetAsyncKeyState(win32con.VK_CONTROL) and win32api.GetAsyncKeyState(win32con.VK_F2):
        #     # CTRL+F1 ,截图
        #     tools.tools_open()
        #
        # if win32api.GetAsyncKeyState(win32con.VK_CONTROL) and win32api.GetAsyncKeyState(win32con.VK_F3):
        #     # CTRL+F1 ,截图
        #     self.btn_local_apps()

    def btn_min_click(self):
        self.showMinimized()

    def btn_close_click(self):
        self.close()

    def btn_hwnd_click(self):
        self.hwnd_window.show()

    def btn_config(self):
        self.btn_local_apps()
        self.config_window = ConfigWindow()
        self.config_window.show()

    def enterEvent(self, event):
        self.__show_menu()
        self.frame.setStyleSheet("background-color: #90000000;border-radius:10px;")

        for btn in self.menubuttons:
            btn.setStyleSheet(
                "QCommandLinkButton:hover{background:#B0000000;} QCommandLinkButton{background:#00000000;color:white;border-radius:0px;min-height:60px;min-width:100px}")
        # self.label_capture.setStyleSheet("QLabel{color:white;background:#90FFFFFF; border-radius:10px;padding:5px}")

        self.btn_min.setStyleSheet(
            "QToolButton:hover{background:#B0000000} QToolButton{background:#00000000;color:white;border-radius:0px;margin-top:10px;}")
        self.btn_close.setStyleSheet(
            "QToolButton:hover{background:#B0000000} QToolButton{background:#00000000;color:white;border-radius:0px;margin-top:10px;}")

        self.labele_quick.setStyleSheet(
            "QLabel{background:#00000000;color:white;border-radius:0px;margin-top:10px;}")

    def leaveEvent(self, event):
        self.__hide_menu()
        self.frame.setStyleSheet("background-color: #00000000;border-radius:10px;")
        # self.__auto_move_side()

    def set_run_statue(self, statue: bool, num: int):
        print('?statue')
        pass

    def ico_down_event(self, event):
        if event.button() == Qt.LeftButton:
            self.ico_down_pos = QCursor.pos()
            self.offset = self.ico.pos() + event.pos()
            self.is_dragging = True

    def ico_move_event(self, event):
        if self.is_dragging:
            mouse_pos = QCursor.pos()
            self.move(mouse_pos.x() - self.offset.x(), mouse_pos.y() - self.offset.y())
            # new_pos = event.pos() - self.offset
            # self.move(new_pos)
            # self.offset = event.pos() - self.frameGeometry().topLeft()
            # print(event)

    def ico_up_event(self, event):
        if event.button() == Qt.LeftButton:
            self.is_dragging = False
            temp_pos = QCursor.pos() - self.ico_down_pos
            if abs(temp_pos.x()) < 5 and abs(temp_pos.y()) < 5:
                self.start_auto_move_side_logic()

    def start_auto_move_side_logic(self):
        self.auto_close_menu_time = 5
        self.__show_menu()

    def __auto_move_side(self):
        pass

    def __hide_menu(self):
        for l in self.menulabels:
            l.setVisible(False)

        self.capture_context_num = 0

        for b in self.menubuttons:
            b.setVisible(False)

    def __show_menu(self):
        self.capture.setText("截图")
        for l in self.menulabels:
            l.setVisible(True)

        for b in self.menubuttons:
            b.setVisible(True)

    def btn_local_apps(self):

        if not os.path.exists(py_apps_quick_start):
            os.mkdir(py_apps_quick_start)

        # 生成快捷方式
        file_list = get_workspace_list()
        for f in file_list:
            pyrun_file = os.path.join(f, "__init__.py")
            if not os.path.exists(pyrun_file):
                pyrun_file = os.path.join(f, "main.py")
                if not os.path.exists(pyrun_file):
                    continue

            exe_file = os.path.join(py_apps_quick_start, os.path.basename(f) + ".bat")
            print(exe_file)
            with open(exe_file, 'w') as exe_bat:
                exe_bat.write("@echo off\n")
                exe_bat.write(f'cmd /k "workon airclick && python {pyrun_file}"')

        # os.startfile(py_apps_quick_start)

    def btn_home(self):
        try:
            self.home_window.show()
            self.home_window.setWindowState(Qt.WindowActive)
        except Exception as e:
            print(str(e))

    def btn_logs(self):
        try:
            BarWindow.log_window.show()
        except Exception as e:
            print(str(e))

    def btn_capture(self):
        # self.capture.setEnabled(False)
        # self.start_auto_move_side_logic()
        self.capture.setVisible(True)
        tools.tool_capture_save()
        self.capture_context_num += 1
        self.capture.setText("截图 " + str(self.capture_context_num))

        # self.capture_progress.setVisible(True)
        # def run_progress():
        #     progress_value = 0
        #
        #     while progress_value < 101:
        #         self.capture_progress.setValue(progress_value)
        #         time.sleep(0.01)
        #         progress_value += 5
        #
        #     self.capture_progress.setVisible(False)
        #
        # threading.Thread(target=run_progress).start()


class HwndWindow(QFrame):
    def __init__(self):
        super().__init__()
        loadUi(os.path.join(current_dir, ui_hwnd_path), self)
        self.setWindowFlag(Qt.WindowStaysOnTopHint)
        self.label_hwnd.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.label_title.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.label_name.setTextInteractionFlags(Qt.TextSelectableByMouse)

        size = pyautogui.size()

        self.move(size.width - self.width() - 10, 10)

    def change_hwnd(self, active_win):
        if active_win:
            self.label_hwnd.setText(str(active_win.hwnd))
            self.label_title.setText(str(active_win.title))
            self.label_name.setText(str(active_win.name))
        else:
            self.label_hwnd.setText('')
            self.label_title.setText('')
            self.label_name.setText('')


class ConfigWindow(QFrame):
    def __init__(self):
        super().__init__()
        loadUi(ui_config_path, self)
        # self.setWindowTitle("")
        self.env_thread = EnvThread()
        self.pkg_thread = PkgThread()
        self.env_thread.statue.connect(self.env_thread_statue)
        if tools.get_workspace():
            self.config_work.setText(tools.get_workspace())
        else:
            self.config_work.setText("请立即设置环境变量")

        self.config_work.clicked.connect(self.btn_config_work)

        self.comboBox.currentIndexChanged.connect(self.current_env_change)
        self.envlist = []
        self.set_env_list()
        self.label_path_exe.setTextInteractionFlags(Qt.TextSelectableByMouse)

        self.btn_menu_app.clicked.connect(self.btn_menu_app_click)
        self.btn_menu_config.clicked.connect(self.btn_menu_config_click)

        # self.pages.setCurrentIndex(1)
        self.set_page(0)
        self.run_satue = [AirIcon("ico_run.png"), AirIcon("ico_stop.png")]

        self.worker = {}

        self.app_chilren_views = []

        self.addData()

    def btn_menu_app_click(self):
        self.set_page(0)

    def btn_menu_config_click(self):
        self.set_page(1)

    def set_page(self, current_page: int):
        pages = [(self.btn_menu_app, self.page_1), (self.btn_menu_config, self.page_2)]

        for i in range(len(pages)):
            if i == current_page:
                self.pages.setCurrentWidget(pages[i][1])
                # 设置样式
                pages[i][0].setStyleSheet(
                    "QCommandLinkButton{min-height:60px;min-width:100px;border:none;border-radius:15px;background:#d1e7e2}")
            else:
                pages[i][0].setStyleSheet(
                    "QCommandLinkButton{min-height:60px;min-width:100px;border:none;border-radius:15px;background:#00000000}QCommandLinkButton:hover{min-height:60px;min-width:100px;border:none;border-radius:15px;background:#d1e7e2}")

    def current_env_change(self, index):
        envdao = self.envlist[index]
        self.env_thread.setkv(tools.ENV_AIRCLICK_VIRTUALENV, envdao['name'])
        self.label_path_exe.setText(envdao['python'])
        self.env_thread.start()

    def set_env_list(self):
        self.envlist = tools.get_virtualenv_list()
        for item in self.envlist:
            if item['is_current']:
                self.comboBox.setCurrentText(item['name'])
                self.label_path_exe.setText(item['python'])

            self.comboBox.addItem(item['name'])

    def btn_config_work(self):
        file_dialog = QFileDialog()
        file_dialog.setFileMode(QFileDialog.Directory)
        if file_dialog.exec_() == QFileDialog.Accepted:
            select_folder = file_dialog.selectedFiles()[0]
            self.config_work.setText(select_folder)
            self.env_thread.setkv(tools.ENV_AIRCLICK_WORKSPACE, select_folder)
            self.env_thread.start()

    def env_thread_statue(self, statue: int):
        if statue == 2:
            print('结束')

    def enterEvent(self, event):
        super().enterEvent(event)
        # self.addData()

    def addData(self):
        # 加载一下
        # self.scrool_apps_contain.r

        try:
            for v in self.app_chilren_views:
                self.app_scroll.removeWidget(v)
                # v.deleteLater()

            self.app_scroll.update()

            self.work_dir = tools.get_workspace()
            self.app_list = []
            # app_scroll
            for filename in os.listdir(self.work_dir):
                file_item = os.path.join(self.work_dir, filename)
                if os.path.isdir(file_item):
                    item_view = self.create_item(file_item)
                    self.app_chilren_views.append(item_view)
                    self.app_scroll.addWidget(item_view)

        except Exception as e:
            print(str(e))

    def create_item(self, file: str):

        app_name = os.path.basename(file)

        def itme_run_click(run_btn):
            print(run_btn, app_name)

            exe_file = os.path.join(py_apps_quick_start, app_name + ".bat")
            win_handle = win32api.ShellExecute(0, 'open', 'cmd', f'/c {exe_file}', '', 1)
            # os.system("D:\\work\\airclick_pylib\\airclick\\windows\\client\\assets\\tools\\apps\\first.bat")

        def itme_package_click(run_btn):
            print(run_btn, app_name)
            pak_dir = os.path.join(tools.get_workspace(), app_name)
            if not self.pkg_thread.isRunning():
                self.pkg_thread.set_package_path(pak_dir)
                self.pkg_thread.start()

        frame = loadUi(os.path.join(current_dir, ui_app_item_path), None)
        frame.label.setText(app_name)

        try:
            if app_name in self.worker:
                if self.worker[app_name].isRunning():
                    frame.run_btn.setIcon(self.run_satue[1])
                else:
                    frame.run_btn.setIcon(self.run_satue[0])
            else:
                frame.run_btn.setIcon(self.run_satue[0])
        except Exception as e:
            print(str(e))

        frame.run_btn.clicked.connect(lambda checked: itme_run_click(frame.run_btn))
        frame.package_btn.clicked.connect(lambda checked: itme_package_click(frame.package_btn))
        return frame


class WorkSpaceWindow(QFrame):
    def __init__(self):
        super().__init__()
        loadUi(os.path.join(current_dir, ui_workspace_path), self)
        self.env_thread = PyEnvSet()


        self.btn_worksapce_choose.clicked.connect(self.worksapce_choose)



    def worksapce_choose(self):
        file_dialog = QFileDialog()
        file_dialog.setFileMode(QFileDialog.Directory)
        if file_dialog.exec_() == QFileDialog.Accepted:
            self.label_info.setText("正在设置中,请稍后...")
            select_folder = file_dialog.selectedFiles()[0]
            self.input_worksaoce_des.setText(select_folder)
            print(select_folder)
            self.env_thread.setkv(tools.ENV_AIRCLICK_WORKSPACE, select_folder)
            self.env_thread.start()
            # self.close()


class HomeWindow(QFrame):
    _instance = None
    log_window = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)

        return cls._instance

    def __init__(self):
        super().__init__()
        loadUi(os.path.join(current_dir, ui_home_path), self)
        self.work_dir = ""
        self.app_list = []
        self.win_workspace = WorkSpaceWindow()
        self.setting_work.clicked.connect(self.fun_setting_work)
        self.setting_python.clicked.connect(self.fun_setting_python)
        self.label_work.setText(check_workspace()[1])
        self.label_py.setText(sys.executable)

        self.run_satue = [AirIcon("ico_run.png"), AirIcon("ico_stop.png")]

        self.worker = {}

    def on_run_statue(self, statue: bool):
        print(statue)
        self.addData()

        run_works = 0

        for work in self.worker:
            if self.worker[work].isRunning():
                run_works += 1

        if run_works > 0:
            BarWindow.get_instance().set_run_statue(True, run_works)
        else:
            BarWindow.get_instance().set_run_statue(False, run_works)

    def on_log_info(self, text):
        BarWindow.log_window.add_log(text)

    def on_log_error(self, text):
        BarWindow.log_window.add_log(text)

    def fun_setting_work(self):
        self.win_workspace.show()

    def fun_setting_python(self):
        self.win_pyinstall.show()

    def enterEvent(self, event):
        super().enterEvent(event)
        self.addData()

    def addData(self):
        # 加载一下
        # self.scrool_apps_contain.r

        while self.scrool_apps_contain.rowCount() > 0:
            self.scrool_apps_contain.takeRow(0)

        workSpace, self.work_dir = check_workspace()
        self.app_list = []
        # scrool_apps_contain
        for filename in os.listdir(self.work_dir):
            file_item = os.path.join(self.work_dir, filename)
            if os.path.isdir(file_item):
                self.scrool_apps_contain.addRow(self.create_item(file_item))
                # pass
                # self.create_item(file_item)

        # self.

    def create_item(self, file: str):

        app_name = os.path.basename(file)

        def tool_click(run_btn):
            print(run_btn)
            if app_name in self.worker:
                runner = self.worker[app_name]
                if runner.isRunning():
                    print('stop')
                    try:
                        runner.stop()
                    except Exception as e:
                        print(str(e))
                else:
                    runner.start()
            else:
                runner = RunThread(file)
                runner.run_statue.connect(self.on_run_statue)
                runner.log_info.connect(self.on_log_info)
                runner.log_error.connect(self.on_log_error)
                self.worker[runner.name] = runner
                runner.start()

        frame = loadUi(os.path.join(current_dir, ui_app_item_path), None)
        frame.label.setText(app_name)

        try:
            if app_name in self.worker:
                if self.worker[app_name].isRunning():
                    frame.run_btn.setIcon(self.run_satue[1])
                else:
                    frame.run_btn.setIcon(self.run_satue[0])
            else:
                frame.run_btn.setIcon(self.run_satue[0])
        except Exception as e:
            print(str(e))

        frame.run_btn.clicked.connect(lambda checked: tool_click(frame.run_btn))
        return frame


class LogWindow(QFrame):
    max_lines = 100

    def __init__(self, parent=None):
        super().__init__(parent)
        loadUi(os.path.join(current_dir, ui_log_path), self)

    def add_log(self, msg: str):
        if self.form_log.rowCount() >= 10:
            self.form_log.removeRow(0)

        l_t = QLabel(self)
        l_t.setText("时间")
        l_msg = QLabel(self)
        l_msg.setText(msg)
        l_msg.setTextInteractionFlags(Qt.TextSelectableByMouse)

        self.form_log.addRow(l_t, l_msg)
        vbar = self.scrollArea.verticalScrollBar()
        vbar.setValue(vbar.maximum())


class RunThread(QThread):
    log_info = pyqtSignal(str)
    log_error = pyqtSignal(str)
    run_statue = pyqtSignal(bool)

    def __init__(self, path_file: str):
        super().__init__()
        self.proc = None
        self.path = path_file
        self.name = os.path.basename(self.path)

    def pause(self):
        if self.proc:
            self.proc.suspend()

    def stop(self):
        if self.proc:
            print('stop')
            # self.proc.terminate()
            os.kill(self.proc.pid, signal.CTRL_C_EVENT)

    def run(self) -> None:
        self.run_statue.emit(True)
        main_file = os.path.join(self.path, 'main.py')
        main_file = main_file.replace('\\', '/')
        # if os.path.exists(main_file):
        #     try:
        #         print(main_file)
        #         with subprocess.Popen(['cmd', 'python', main_file], stdout=subprocess.PIPE,
        #                               stderr=subprocess.PIPE) as self.proc:
        #             for line in self.proc.stdout:
        #                 print("?-" + line.decode('utf-8').strip())  # 打印标准输出的一行
        #                 self.log_info.emit(line.decode('utf-8').strip())
        #
        #             for line in self.proc.stderr:
        #                 # log_win.add_log(line.decode('utf-8').strip())
        #                 print(line.decode('utf-8').strip())  # 打印标准错误的一行
        #                 self.log_error.emit(line.decode('utf-8').strip())
        #     except Exception as e:
        #         print(str(e))

        # self.exec(f'python {main_file}\n')

        os.startfile(os.path.dirname())

        # self.proc = subprocess.Popen(["cmd.exe"],shell=True)

        self.name = None
        self.run_statue.emit(False)

    def exec(self, cmd: str):

        try:
            with subprocess.Popen("cmd", shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE, encoding='cp936') as self.proc:

                self.proc.stdin.write(cmd)
                self.proc.stdin.flush()
                self.proc.stdin.close()

                for line in self.proc.stdout:
                    print(line.strip())
                    # self.proc.terminate()
                    # self.log_info.emit(line.strip())

                for line in self.proc.stderr:
                    print(line.strip())
                    # self.log_error.emit(line.strip())

            return [self.proc.returncode]
        except Exception as e:
            print(str(e))


def enter():
    try:
        app = QApplication(sys.argv)

        step3 = check_workspace()

        # 检测完毕后，再打开悬浮窗口
        if step3:
            bar_window = BarWindow()
            bar_window.show()
        else:
            def mysi(istrue):
                print('123')
                if workspace_win:
                    workspace_win.close()
                    bar_window = BarWindow()
                    bar_window.show()

            workspace_win = WorkSpaceWindow()
            workspace_win.env_thread.set_statue.connect(mysi)
            workspace_win.show()

        sys.exit(app.exec_())
    except Exception as e:
        print(str(e))


def choose_andsetting_env(intput_view, callback):
    def child_run():
        try:
            res = subprocess.run([select_file, "--version"])
            if res:
                select_dir = os.path.dirname(select_file)
                select_dir = select_dir.replace('/', '\\')
                # 设置环境变量
                # Machine 是系统，User 是用户
                subprocess.run(['powershell',
                                f'[System.Environment]::SetEnvironmentVariable("Path","{select_dir}" +";" + $env:Path, [System.EnvironmentVariableTarget]::Machine)'])
                subprocess.run(['echo', '%PATH%'], shell=True)

                # 设置当前进程环境变量
                current_path = os.environ['PATH']
                os.environ['PATH'] = select_dir + ";" + current_path
                callback(True, "")
        except Exception as e:
            callback(False, str(e))

    file_dialog = QFileDialog()
    if file_dialog.exec_() == QDialog.Accepted:
        select_file = file_dialog.selectedFiles()[0]
        intput_view.setText(select_file)
        threading.Thread(target=child_run).start()


def set_workspace(path: str):
    try:
        work_config_file = os.path.join(current_dir, py_work_config)
        config = configparser.ConfigParser()
        config['workspace'] = {'path': path}
        with open(work_config_file, 'w') as configfile:
            config.write(configfile)

        return True
    except Exception as e:
        print(str(e))
        return False


def check_workspace():
    if tools.get_workspace():
        return True
    else:
        return False


def get_workspace_list():
    work_dir = tools.get_workspace()
    app_list = []
    # scrool_apps_contain
    if work_dir:
        for filename in os.listdir(work_dir):
            file_item = os.path.join(work_dir, filename)
            if os.path.isdir(file_item):
                app_list.append(file_item)

    return app_list



from airclick.action.path import path
from typing import List
from airclick.action.path import path


def perform(*paths: List[path]) -> None:
    """
    执行手势操作

    :param paths: 需要执行的路径对象列表
    """
    pass



import os.path
import subprocess
import sys
import time
import webbrowser

from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QPushButton
from airclick.windows import screen

ENV_AIRCLICK_WORKSPACE = "AIRCLICK_WORKSPACE"
ENV_AIRCLICK_VIRTUALENV = "AIRCLICK_VIRTUALENV"
ENV_HOME = "WORKON_HOME"


def tools_open():
    webbrowser.open("http://127.0.0.1:8080/colors")


def tool_capture_save() -> str:
    # pass
    img = screen.capture()

    current_dir = os.path.dirname(os.path.realpath(sys.argv[0]))

    # img.show()
    filename = os.path.join(current_dir, "assets/tools/capture/" + str(int(time.time())) + ".png")
    print(filename)
    if not os.path.exists(os.path.dirname(filename)):
        os.makedirs(os.path.dirname(filename), exist_ok=True)
    img.save(filename)
    return filename
    # img.save("./assets/tools/capture/"+time.time()+".png")


class ToolButton(QPushButton):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.arg = {}


class EnvThread(QThread):
    statue = pyqtSignal(int)

    def __init__(self):
        super().__init__()
        self.key = None
        self.value = None

    def setkv(self, k, v):
        self.key = k
        self.value = v

    def run(self):
        self.statue.emit(1)
        set_env(key=self.key, value=self.value)
        self.statue.emit(2)


def set_env(key: str, value: str) -> bool:
    try:
        # Machine 是系统，User 是用户
        subprocess.run(['powershell',
                        f'[System.Environment]::SetEnvironmentVariable("{key}","{value}", [System.EnvironmentVariableTarget]::User)'])
        subprocess.run(['echo', f'%${key}%'], shell=True)
        os.environ[f'{key}'] = value
        return True
    except Exception as e:
        print(str(e))
        return False


def get_workspace():
    return os.environ.get(ENV_AIRCLICK_WORKSPACE)


def set_workspace(path: str):
    return set_env(ENV_AIRCLICK_WORKSPACE, path)


def get_virtualenv_cureent():
    return os.environ.get(ENV_AIRCLICK_VIRTUALENV)


def get_virtualenv_list():
    vs_list = []
    v_home = os.environ.get(ENV_HOME)
    v_current_home = os.environ.get(ENV_AIRCLICK_VIRTUALENV)
    if v_home is None:
        v_home = os.path.expanduser("~\\Envs")
        print(v_home)

    if v_current_home is None:
        v_current_home = "airclick"

    vs = os.listdir(v_home)

    for v in vs:
        exe_file = os.path.join(v_home, v, "Scripts", 'python.exe')
        # print(exe_file)
        if os.path.exists(exe_file):
            is_current = v_current_home == v
            vdao = {'name': os.path.basename(v), 'python': exe_file, 'is_current': is_current}
            vs_list.append(vdao)

    return vs_list



import os.path
import shutil
import sys
from urllib.parse import unquote
from PIL import Image
from flask import Flask, render_template
import io
import json
from PyQt5.QtCore import QByteArray, QBuffer
from flask import app, request, Response
from airclick.windows.screen import FindColors, Ocr, FindImages, CompareColors
from airclick.windows.window import Window
from airclick.windows.client import tools
from airclick.windows.client.dao.server_dao import MFile, Result
from gevent import pywsgi
from airclick.windows.client.ui import get_workspace_list

current_dir = os.path.dirname(os.path.realpath(sys.argv[0]))

template_folder = os.path.join(current_dir, 'assets\\templates')
static_folder = os.path.join(current_dir, 'assets')
print(template_folder, static_folder)
static_url_path = '../../../airclick/assets'

app = Flask(__name__, template_folder=template_folder,
            static_folder=static_folder)


@app.route("/")
def page_home():
    return render_template("index.html")


@app.route("/hwnd")
def page_hwnd():
    return render_template("hwnd.html")


@app.route("/colors")
def page_colors():
    return render_template("colors.html")


@app.route("/api/tool/hwnd", methods=['POST'])
def api_tool_hwnd():
    sel_dict = request.json
    print(json.dumps(sel_dict))
    nodes = Window.Selector(dict=sel_dict).exec_find()
    return json.dumps([node.to_dict() for node in nodes])


@app.route("/api/tool/capture")
def api_tool_capture():
    hwnd = request.args.get('hwnd')
    if hwnd:
        win = Window(int(hwnd))
        img = win.capture()
        if img:
            byte_array = QByteArray()
            buffer = QBuffer(byte_array)
            img.save(buffer, "PNG")
            byte_stream = bytes(byte_array)
            return Response(byte_stream, mimetype='image/png')
        else:
            return '无效图像'
    else:

        #  PIL Image
        img = Window.capture()
        byte_strem = io.BytesIO()
        img.save(byte_strem, format='PNG')
        byte_strem = byte_strem.getvalue()

        return Response(byte_strem, mimetype='image/png')


@app.route("/api/tool/capture/list", methods=['GET'])
def api_tool_capture_list():
    try:
        need_capture = request.args.get("capture")
        print(need_capture)
        if need_capture and need_capture == 'true':
            tools.tool_capture_save()
        capture_dir = os.path.join(current_dir, "assets", "tools", "capture")
        all_pic = os.listdir(capture_dir)

        res = Result(0, "")
        tool_capture_list = []
        for f in all_pic:
            tool_capture_list.append(MFile(os.path.join(capture_dir, f)))

        res.set_data([f.to_dict() for f in tool_capture_list])

        res.data.sort(key=lambda x: x['last_modified'], reverse=True)

        return json.dumps(res.to_dict())
    except Exception as e:
        return json.dumps(Result(0, str(e)).to_dict())


@app.route("/api/file/getpicture", methods=['GET'])
def api_tool_file_getimage():
    file_path = request.args.get("path")
    rect_str = request.args.get("rect")
    max_height = request.args.get("max_height")
    max_width = request.args.get("max_width")
    save_path = request.args.get("save")
    save_convert = request.args.get("ysave")
    rect = None

    if rect_str:
        print(rect_str)
        rect = json.loads(rect_str)

    if file_path:
        img = Image.open(file_path)

        if rect:
            img = img.crop([rect['left'], rect['top'], rect['right'], rect['bottom']])

        if max_height or max_width:
            aspect_ratio = img.size[0] / img.size[1]
            if max_width:
                target_width = int(max_width)
                target_height = int(aspect_ratio * target_width)
                img = img.resize((target_width, max_height))
            elif max_height:
                target_height = int(max_height)
                target_width = int(aspect_ratio * max_height)
                img = img.resize((target_width, target_height))

    byte_strem = io.BytesIO()
    img.save(byte_strem, format='PNG')
    byte_strem = byte_strem.getvalue()

    if save_path:
        save_path = unquote(save_path)
        save_path = save_path.replace('\\', '/')

        if os.path.exists(save_path) and save_convert == 'false':
            print('文件已存在')
            return json.dumps(Result(-1, msg='文件已存在').to_dict())
        else:
            parent_dir = os.path.dirname(save_path)
            if not os.path.exists(parent_dir):
                os.makedirs(parent_dir)
            img.save(save_path, format='PNG')

    return Response(byte_strem, mimetype='image/png')


@app.route("/api/file/remove", methods=['GET'])
def api_tool_file_remove():
    file_path = request.args.get("path")
    relative_path = request.args.get("relative")
    target_file = file_path

    if relative_path:
        target_file = os.path.join(current_dir, file_path)
    else:
        target_file = unquote(target_file)

    print(target_file)
    if os.path.isdir(target_file):
        shutil.rmtree(target_file)
    else:
        os.remove(target_file)

    return json.dumps(Result().to_dict())


@app.route("/api/fun/findcolors", methods=['POST'])
def api_fun_findcolors():
    sel_dict = request.json
    print(json.dumps(sel_dict))

    points = FindColors(dict_p=sel_dict).exec_find()
    print(points)

    res = Result(0, "")
    res.set_data([p.to_dict() for p in points])

    return json.dumps(res.to_dict())


@app.route("/api/fun/comparecolors", methods=['POST'])
def api_fun_comparecolors():
    sel_dict = request.json
    print(json.dumps(sel_dict))

    is_equ = CompareColors(dict_p=sel_dict).compare()
    print(is_equ)

    res = Result(0, "")

    res.set_data(is_equ)

    return json.dumps(res.to_dict())


@app.route("/api/fun/findimages", methods=['POST'])
def api_fun_findimages():
    sel_dict = request.json
    print(json.dumps(sel_dict))

    find_imgs = FindImages(dict_p=sel_dict)
    points = None

    if 'findtype' in sel_dict:
        if sel_dict['findtype'] == 'find':
            points = [find_imgs.find()]

        if sel_dict['findtype'] == 'find_all':
            points = find_imgs.find_all(0)

        if sel_dict['findtype'] == 'find_template':
            points = [find_imgs.find_template()]

        if sel_dict['findtype'] == 'find_all_template':
            points = find_imgs.find_all_template()

        if sel_dict['findtype'] == 'find_sift':
            points = [find_imgs.find_sift()]

        if sel_dict['findtype'] == 'find_all_sift':
            points = find_imgs.find_all_sift()

    res = Result(0, "")

    print(points)
    res.set_data(points)

    return json.dumps(res.to_dict())


@app.route("/api/fun/ocr", methods=['POST'])
def api_fun_ocr():
    sel_dict = request.json
    print(json.dumps(sel_dict))

    points = Ocr(dict_p=sel_dict).exec_find_paddle()

    res = Result(0, "")
    if points:
        res.set_data([p.to_dict() for p in points])

    return json.dumps(res.to_dict())


@app.route("/api/model/list", methods=['POST'])
def api_model_list():
    app_list = get_workspace_list()
    file_list = []

    for app in app_list:
        file_list.append(MFile(app))

    res = Result(0, "")
    res.set_data([p.to_dict() for p in file_list])

    return json.dumps(res.to_dict())


def run():
    server = pywsgi.WSGIServer(('127.0.0.1', 8080), app)
    server.serve_forever()


def close():
    print("close")



from typing import Union, List

from airclick.node.node import Node


class Selector:

    @staticmethod
    def cache(is_cache:bool):
        pass

    def __init__(self, mode: int = 0):
        pass

    def find(self) -> Node:
        pass

    def until_find(self) -> Node:
        pass

    def find_all(self) -> List[Node]:
        pass

    def until_find_all(self) -> List[Node]:
        pass

    def id(self, nid: str) -> 'Selector':
        pass

    def text(self, ntext: str) -> 'Selector':
        pass

    def type(self, ntype: str) -> 'Selector':
        pass

    def desc(self, ndesc: str) -> 'Selector':
        pass

    def hintText(self, nhintText: str) -> 'Selector':
        pass

    def packageName(self, npackageName: str) -> 'Selector':
        pass

    def path(self, pathstr: str) -> 'Selector':
        pass

    def inputType(self, ntype: int) -> 'Selector':
        pass

    def depth(self, ntype: int) -> 'Selector':
        pass

    def drawingOrder(self, ntype: int) -> 'Selector':
        pass

    def childCount(self, nccount: int) -> 'Selector':
        pass

    def inputType(self, ninputtype: int) -> 'Selector':
        pass

    def maxTextLength(self, nmax: int) -> 'Selector':
        pass

    def clickable(self, nclickable: bool) -> 'Selector':
        pass

    def checkable(self, ncheckable: bool) -> 'Selector':
        pass

    def checked(self, nchecked: bool) -> 'Selector':
        pass

    def editable(self, neditable: bool) -> 'Selector':
        pass

    def enabled(self, nenabled: bool) -> 'Selector':
        pass

    def dismissable(self, ndismissable: bool) -> 'Selector':
        pass

    def focusable(self, nfocusable: bool) -> 'Selector':
        pass

    def focused(self, nfocused: bool) -> 'Selector':
        pass

    def longClickable(self, nlongClickable: bool) -> 'Selector':
        pass

    def visible(self, nvisible: bool) -> 'Selector':
        pass

    def parent(self, *n: Union[float, int]) -> 'Selector':
        """
        获取父控件

        :param n: 获取第n个父元素,默认获取所有父元素
                        (2):获取爷爷元素
                        (3):获取太爷爷元素
                        (1,3):获取第1和第3个父元素
                        (1.3):获取第1-3 之间的所有父元素
        """
        pass

    def child(self, *n: Union[float, int]) -> 'Selector':
        """
        获取孩子控件

        :param n:可以填写多个数字
                不填写任何参数,获取所有孩子控件;
                当数字为正整数(例如1):获取第1个孩子控件;
                当数字为负整数(例如:-1) 获取倒数第1个孩子;
                当数字为正小数(例如:1.3):获取1-3之间的所有孩子;
                当数字为负小数(例如:-1.3):获取倒数 1-3之间的所有孩子;
        """
        pass

    def brother(self, *n: Union[float, int]) -> 'Selector':
        """
        获取兄弟控件

        :param n: 获取第n个兄弟控件|
                () 默认不填:获取所有兄弟控件|
                (1) 获取第一个兄弟控件 |
                (1,2) 获取第1和第2个兄弟控件|
                (1.4) 获取1-4之间的所有兄弟控件|
                (0.1) 获取当前控件的下一个兄弟控件|
                (-0.1) 获取当前空间的上一个兄弟控件|
                (-1) 获取倒数第1个兄弟控件
        """
        pass





from typing import Union

from airclick import graphics


# 点击
def click(x: Union[int, graphics.Point], y=None, dur=20) -> None:
    """
    点击屏幕某个位置
    可传入一组坐标，也可以传入 一个Point对象
    """
    if isinstance(x, graphics.Point):
        point = x
        x = point.x
        y = point.y
    elif y is None:
        raise ValueError("If the first argument is not a Point, both x and y should be provided.")


# 滑动
def slide(x: int, y: int, x1: int, y1: int, dur=20) -> None:
    pass


# 输入
def input(data: str) -> None:
    pass


# 捕获动作
class Catch:
    def click(self):
        return self

    def msg(self, message: str):
        return self

    def shine(self, isShine: bool):
        return self


class hid:
    @staticmethod
    def click(x: int, y: int, dur: int = 20):
        pass

    @staticmethod
    def slide(x: int, y: int, x1: int, y1: int):
        pass

    @staticmethod
    def key(*key: str):
        pass



# 手势-创建路径
from typing import Self


class path:
    def __init__(self, startTime=0, duration=20, willContinue=False):
        """
        创建一个新的路径对象

        参数:
        - startTime (number): 延迟多久开始绘制路径，单位ms，默认为0
        - duration (number): 路径绘制的时长，单位ms，默认为20
        - willContinue (boolean): 在路径结束后，手指是否抬起，默认为False
        """

    def moveTo(self, x:int, y:int) -> None:
        """
        移动初始点到指定位置

        参数:
        - x (number): X轴坐标
        - y (number): Y轴坐标
        """
        pass

    def lineTo(self, x: int, y: int)-> None:
        """
        画直线到指定位置

        参数:
        - x (number): X轴坐标
        - y (number): Y轴坐标
        """
        pass

    def quadTo(self, x1: int, y1:int, x2:int, y2:int)-> None:
        """
        使用二次贝塞尔曲线绘制路径

        参数:
        - x1 (number): 控制点1的X轴坐标
        - y1 (number): 控制点1的Y轴坐标
        - x2 (number): 控制点2的X轴坐标
        - y2 (number): 控制点2的Y轴坐标
        """
        pass







import airclick.windows.client.server as server
import airclick.windows.client.ui as ui
import threading


def startServer():
    server.run()

if __name__ == '__main__':
    thread_server = threading.Thread(target=startServer)
    thread_server.start()
    ui.enter()

# home = ui.HomeWindow()
# home.show()


# ui.check_workspace()



from typing import List

from airclick import Selector


class Rect:
    def __init__(self):
        self.left = 0
        self.top = 0
        self.right = 0
        self.bottom = 0
        pass

    def width(self):
        return 0;

    def height(self):
        return 0;

    def centerX(self):
        return 0;

    def centerY(self):
        return 0;


class Node:
    def __int__(self):
        self.id = ""
        self.text = ""
        self.type = ""
        self.desc = ""
        self.hintText = ""
        self.packageName = ""
        self.rect = Rect()
        self.childCount = 0
        self.inputType = 0
        self.maxTextLength = 0
        self.clickable = True
        self.checkable = True
        self.checked = True
        self.editable = True
        self.enabled = True
        self.visible = True
        self.dismissable = True
        self.focusable = True
        self.focused = True
        self.longClickable = True

        pass

    def find(self, sel: Selector) -> 'Node':
        return self

    def find_all(self, sel: Selector) -> 'List[Node]':
        return [self]

    def click(self):
        return True

    def long_click(self):
        return True

    def slide(self, ori: int):
        """
        滑动

        :param ori: 滑动的方向;
                    -1:向前滑动;
                    1:向后滑动
        """
        return True

    def input(self, msg: str):
        """
        输入信息

        :param msg: 输入的信息
        """
        return True





import abc
from typing import Any


class R:
    def __init__(self, path: str) -> None:
        pass

    def sd(self, childpath: str = None) -> str:
        pass

    def context(self):
        """
        获取 android 工程上下文
        """
        pass

    def root(self, childpath: str= None) -> str:
        pass

    def res(self, childpath: str = None) -> str:
        pass

    def ui(self, file: str = None) -> str:
        pass

    def exit(self) -> str:
        pass

    def reboot(self) -> str:
        pass


class Device:

    def id(self) -> 'str':
        pass

    def display(self) -> Any:
        pass

    def name(self) -> str:
        pass

    def brand(self) -> str:
        pass

    def model(self) -> str:
        pass

    def sdk(self) -> str:
        pass

    def version(self) -> str:
        pass

    def ip(self) -> str:
        pass

    def currentAppInfo(self) -> Any:
        pass


class MediaPlayListener(metaclass=abc.ABCMeta):
    def prepare(self, player: Any):
        """
        当音频准备完成后回掉该方法

        调用 player.start() 可播放音频
        调用 player.getDuration() 可获取音频的长度
        """
        pass

    def completion(self):
        """
        音频播放完毕后回调该方法
        """
        pass


class Media:
    @staticmethod
    def volume(percent: int, ptype: int):
        """
        调节音量大小

        :param percent: 音量大小 1-100之间
        :param ptype: 音量类型,默认3; 3 = 音乐回放即媒体音量;5 = 窗口顶部状态栏Notification;4= 警告; 2= 铃声;1 = 系统; 0 =通话
        """
        pass

    @staticmethod
    def talk(text: str):
        """
        语音朗读

        :param text: 要朗读的文字
        """
        pass

    @staticmethod
    def play(path: str, listener: MediaPlayListener):
        """
        播放音频文件
        """
        pass

    @staticmethod
    def recode(path: str, time: int = None):
        """
        录制音频文件
        """
        pass

    @staticmethod
    def vibrate(time: int):
        """
        设备震动
        """
        pass

class Clipboard:
    @staticmethod
    def put(msg: str):
        """
        将数据放入剪贴板
        """
        pass

    @staticmethod
    def get():
        """
        读取剪贴板数据
        """
        pass


class Channel:
    def __init__(self, listener: Any):
        pass



from typing import Any
from typing import List, Optional, Tuple
from airclick.graphics import Point
from airclick.system import R


class Bitmap:
    def __init__(self, width: int, height: int) -> None:
        self.width = width
        self.height = height
        self.pixels: List[List[Tuple[int, int, int]]] = [[(0, 0, 0)] * width for _ in range(height)]

    def set_pixel(self, x: int, y: int, color: Tuple[int, int, int]) -> None:
        self.pixels[y][x] = color

    def get_pixel(self, x: int, y: int) -> Tuple[int, int, int]:
        return self.pixels[y][x]

    # 其他方法和属性的类型注释
    # ...


class Screen:

    @staticmethod
    def cache(is_cache:bool):
        pass

    @staticmethod
    def bitmap(x: int = None, y: int = None, x1: int = None, y1: int = None) -> Bitmap:
        """
        将屏幕指定区域截取为Bitmap可操作的对象

        参数:
        - x (int): 起始点横坐标 (可选)
        - y (int): 起始点纵坐标 (可选)
        - x1 (int): 终止点横坐标 (可选)
        - y1 (int): 终止点纵坐标 (可选)

        返回结果:
        - Bitmap: Java中的Bitmap对象，详细属性请参考：https://developer.android.google.cn/reference/kotlin/android/graphics/Bitmap?hl=en
        """

    @staticmethod
    def file2Bitmap(path: str, sampleSize: int = None) -> Bitmap:
        """
        将图片文件读取为内存图像Bitmap

        参数:
        - path (str): 图片路径地址
        - sampleSize (int): 图像缩放参数 (可选)

        返回结果:
        - Bitmap: Java中的Bitmap对象，详细属性请参考：https://developer.android.google.cn/reference/kotlin/android/graphics/Bitmap?hl=en
        """
    @staticmethod
    def toFile(path:str,bitmap:Bitmap,quality:int):
        pass

    @staticmethod
    def capture(path: str, bitmap: Bitmap = None, quality: int = None) -> Any:
        """
        截图到指定的文件地址

        参数:
        - path (str): 存储的路径
        - bitmap (Bitmap): Android图像 (可选，默认全屏截图)
        - quality (int): 截图的清晰度 (可选，默认为100，原图)

        返回结果:
        - File: Java中的File对象，详细属性请参考：https://tool.oschina.net/uploads/apidocs/jdk-zh/java/io/File.html
        """

    @staticmethod
    def base64(bitmap:Bitmap):
        pass

    @staticmethod
    def maxside(bitmap:Bitmap,max_side_len):
        pass


class FindColors:
    def __init__(self, colors: str) -> None:
        pass

    def rect(self, x: int, y: int, x1: int, y1: int) -> 'FindColors':
        pass

    def space(self, num: int) -> 'FindColors':
        pass

    def ori(self, num: int) -> 'FindColors':
        pass

    def find(self) -> Optional[Point]:
        pass

    def find_all(self) -> List[Point]:
        pass


class FindImages:
    def __init__(self, part_img: str) -> None:
        pass

    def confidence(self, num: float) -> 'FindImages':
        pass

    def find(self) -> Optional[dict]:
        pass

    def find_all(self) -> List[dict]:
        pass

class GetColorNum:
    def __int__(self):
        pass

    def colors(self,strcolor:str):
        pass

    def rect(self,l,t,r,b):
        pass

    def sim(self,str_rgb:str):
        pass

    def find(self):
        pass


class Ocr:
    def __init__(self, model_path: str = "") -> None:
        """
        如果不传模型,默认会下载ch_PPOCRV3,
        如果传入模型地址:那么模型地址子目录中必须包含7个文件
        "cls.pdiparams"
            ,"cls.pdmodel"
            ,"det.pdiparams"
            ,"det.pdmodel"
            ,"rec.pdiparams"
            ,"rec.pdmodel"
            ,"keys.txt"
        更多模型下载:https://github.com/PaddlePaddle/PaddleOCR/blob/release/2.6/doc/doc_ch/models_list.md
        """
        pass

    def mode(self,mode:int):
        pass

    def rect(self, x: int, y: int, x1: int, y1: int) -> 'Ocr':
        pass

    def pattern(self, regx: str) -> 'Ocr':
        pass

    def max_side_len(self,max):
        pass

    def precision(self,num):
        pass

    def file(self, path: str) -> 'Ocr':
        pass

    def bitmap(self, bitmap: Any) -> 'Ocr':
        pass

    def max_side_len(self, max_side_len: int = 0) -> 'Ocr':
        """
        设置最大边
        如1200,那么传入的资源图 会缩放导1200px,再传给引擎识别
        """
        pass

    def quality(self, quality: int = 16) -> 'Ocr':
        """
        识别精度,默认是16,如果是量化模型,可传入8
        """
        pass

    def find(self) -> Optional[dict]:
        pass

    def find_all(self) -> List[dict]:
        pass


# Callable[[bool], None]
class CompareColors:
    def __init__(self, colors: str) -> None:
        pass

    def diff(self, color: str) -> 'CompareColors':
        pass

    def until(self) -> 'CompareColors':
        pass

    def asy(self, method) -> 'CompareColors':
        pass

    def compare(self) -> bool:
        pass


class QRcode:
    def __init__(self) -> None:
        pass

    def file(self, path: R) -> 'QRcode':
        pass

    def bitmap(self, bitmap: Bitmap) -> 'QRcode':
        pass

    def find(self) -> Any:
        pass

    def rect(self, x: int, y: int, x1: int, y1: int) -> 'QRcode':
        pass


class yolo_v5_res:
    def __init__(self):
        self.x = 0.0
        self.y = 0.0
        self.w = 0.0
        self.h = 0.0
        self.label = ""
        self.prob = 0.0


class yolo_v5:
    def __init__(self, model: str) -> None:
        """
        初始化yolov5 模型

        :param model:可传入模型名称(模型市场中的),也可以传入本地模型路径.
        """
        pass

    def find_all(self) -> 'List[yolo_v5_res]':
        pass



from typing import Any, Union


class Window:
    def __init__(self, layout: str, tunnel: Any = None):
        pass

    def model(self, model: int):
        pass

    def width(self, size: Union[int, str]):
        pass

    def height(self, size: Union[int, str]):
        pass

    def background(self, bgcolor: str):
        pass

    def drag(self, isDrag: bool):
        pass

    def dimAmount(self, dim: float):
        """
        数值在 0-1之间
        0 :完全没有遮罩,1:完全黑的遮罩,0.5:半透明遮罩,默认为0.5
        """
        pass

    def gravity(self, g: int):
        """
        居中位置（引力） 该方法会影响 x，y 函数 的相对位置
        """
        pass

    def x(self, x: int):
        pass

    def y(self, y: int):
        pass

    def show(self):
        pass

    def close(self):
        pass

    def call(self, func: Any):
        pass

    def tunner(self, func: Any):
        pass


class Float:
    def hide(self):
        pass

    def show(self, x: Union[int, float], y: Union[int, float]):
        pass



import ctypes
import sys
import pyautogui
import win32con
import win32gui
from PIL import ImageGrab, Image, ImageQt
import re

from PyQt5.QtWidgets import QApplication

app = QApplication(sys.argv)


class Window:

    @staticmethod
    def active():
        return Window(win32gui.GetForegroundWindow())

    @staticmethod
    def screen_size():
        return pyautogui.size()

    def __init__(self, hwnd: int, fill_children: bool = True):
        self.qapp = None
        self.children = None
        self.hwnd = hwnd
        self.title = win32gui.GetWindowText(hwnd)
        self.rect = win32gui.GetWindowRect(hwnd)
        self.name = win32gui.GetClassName(hwnd)
        self.width = self.rect[2] - self.rect[0]
        self.height = self.rect[3] - self.rect[1]
        # 是否最小化
        if fill_children:
            self.__fill_childs(hwnd)

    def __fill_childs(self, hwnd: int):
        self.children = []

        def node_enum(hwnd, extra):
            self.children.append(Window(hwnd))

        win32gui.EnumChildWindows(hwnd, node_enum, None)

    def __str__(self):
        return {'hwnd': self.hwnd, 'title': self.title, 'name': self.name, 'rect': self.rect,
                'active': self.is_active()}.__str__()

    def to_dict(self):
        tempchilds = []

        if self.children:
            for cnode in self.children:
                tempchilds.append(cnode.to_dict())

        return {
            'hwnd': self.hwnd,
            'name': self.name,
            'title': self.title,
            'rect': self.rect,
            'children': tempchilds,
            "placement": self.placement(),
            # "is_maximized": self.is_maximized(),
            "is_active": self.is_active()
        }

    def capture(self) -> Image:
        return pyautogui.screenshot()

    def views(self):
        pass

    def is_active(self) -> bool:
        return is_active(self)

    # def has_focus(self):
    #     return win32gui.GetFocus() == self.hwnd

    def placement(self):
        # return ctypes.windll.user32.IsIconic(self.hwnd) != 0
        return win32gui.GetWindowPlacement(self.hwnd)[0]

    def frame(self, x=None, y=None, w=None, h=None):
        frame(self, x, y, w, h)

    def mize_mini(self):
        mize_mini(self)

    def mize_max(self):
        mize_max(self)

    def mize_normal(self):
        mize_normal(self)

    def close(self):
        win32gui.SendMessage(self.hwnd, win32con.WM_CLOSE, 0, 0)

    def __base_re(self, pattern, value):
        if not re.match(pattern, value):
            return False

        return True


def find(
        active: bool = True,
        title: str = None,
        name: str = None,
        hwnd: int = None):
    if hwnd:
        return Window(hwnd)

    window_list = find_all(active, title, name)
    if window_list and len(window_list) > 0:
        return window_list[0]

    return None


def find_all(active: bool = None, title: str = None, name: str = None):
    window_list = []

    def tempcall(hwnd, list):
        window_list.append(Window(hwnd, False))

    win32gui.EnumWindows(tempcall, window_list)

    f_window_list = []
    for w in window_list:

        if active is not None:
            if active != w.is_active():
                continue

        if title is not None:
            if not w.__base_re(title, w.title):
                continue

        if name is not None:
            if not w.__base_re(name, w.title):
                continue

        f_window_list.append(w)

    return f_window_list


def frame(win: Window, x=None, y=None, w=None, h=None):
    if x is None:
        x = win.rect[0]

    if y is None:
        y = win.rect[1]

    if w is None:
        w = win.rect[2] - win.rect[0]

    if h is None:
        h = win.rect[3] - win.rect[1]
    win32gui.MoveWindow(win.hwnd, int(x), int(y), int(w), int(h), True)


def is_active(win: Window) -> bool:
    return win32gui.GetForegroundWindow() == win.hwnd


def mize_mini(win: Window):
    win32gui.ShowWindow(win.hwnd, win32con.SW_MINIMIZE)


def mize_max(win: Window):
    win32gui.ShowWindow(win.hwnd, win32con.SW_MAXIMIZE)


def mize_normal(win: Window):
    win32gui.ShowWindow(win.hwnd, win32con.SW_NORMAL)


def close(self):
    win32gui.SendMessage(self.hwnd, win32con.WM_CLOSE, 0, 0)



import os
from typing import Any
from urllib.parse import quote


class Result:
    def __init__(self, code: int = 0, msg: str = ""):
        self.data = None
        self.code = code
        self.msg = msg

    def set_data(self, data: Any):
        self.data = data

    def to_dict(self):
        return {
            "code": self.code,
            "msg": self.msg,
            "data": self.data
        }


class MFile:
    def __init__(self, path: str):
        self.path = path
        self.name = os.path.basename(path)
        self.length = os.path.getsize(path)
        self.last_modified = os.path.getmtime(path)
        self.is_file = os.path.isfile(path)

    def to_dict(self):
        return {
            "path": quote(self.path),
            "name": self.name,
            "length": self.length,
            "last_modified": self.last_modified,
            "is_file": self.is_file
        }





from setuptools import setup, find_packages
from pip._internal.req import req_file
from setuptools.glob import glob

VERSION = '0.0.3.26'
DESCRIPTION = 'airclick 相关python包'
requirments = req_file.parse_requirements('requirements.txt', session='hack')
instll_requires = [req.requirement for req in requirments]

print(instll_requires)

setup(
    name="airclick",
    version=VERSION,
    author="aojoy",
    author_email="aojoytec@163.com",
    description=DESCRIPTION,
    include_package_data=True,

    long_description_content_type="text/markdown",
    long_description=open('README.md', encoding="UTF8").read(),
    packages=find_packages(),
    keywords=['python', "airclick"],
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://airscript.cn/",
    install_requires=instll_requires,
)



import os
import sys
import webbrowser


def open():
    pass


def web_open(url: str):
    webbrowser.open(url)


class R:
    _instance = None

    def __new__(cls, root: str=None):
        if not cls._instance:
            cls._instance = super().__new__(cls)
            cls._instance.root = root
        return cls._instance

    def file(self, child_path: str):
        child_path = child_path.replace('/', '\\')
        print(self.root)
        return os.path.join(self.root, child_path)

    @staticmethod
    def current():
        return os.path.dirname(os.path.realpath(sys.argv[0]))

    @staticmethod
    def exit():
        sys.exit(0)



from typing import Any

from airclick.ui import Window


def toast(msg: str, duration: int = 3, x=-1, y=-1) -> None:
    pass


def alert(msg: str, submit: str) -> None:
    pass


class confirm:
    def __init__(self, msg: str):
        pass

    def title(self, msg: str) -> 'confirm':
        pass

    def submit(self, msg: str) -> 'confirm':
        pass

    def cancel(self, msg: str) -> 'confirm':
        pass

    def close(self):
        pass

    def show(self, pyfun: Any):
        pass


class promat:
    def __init__(self, msg: str):
        pass

    def title(self, msg: str) -> 'promat':
        pass

    def value(self, msg: str) -> 'promat':
        pass

    def hint(self, msg: str) -> 'promat':
        pass

    def submit(self, msg: str) -> 'promat':
        pass

    def cancel(self, msg: str) -> 'promat':
        pass

    def close(self):
        pass

    def show(self, pyfun: Any = None):
        pass


class loger(Window):
    def __init__(self, tuner: Any = None, layout: str = None):
        """
        loger 继承自Window

        @param tuner: 消息通道
        @param layout: 自定义布局(注意该 布局必须包含给定模版中的 js方法.)
        """
        super().__init__(layout)

    def show(self, pyfun: Any = None):
        pass
